<?xml version="1.0" encoding="UTF-8"?>
<gui name="lskat"
     version="3"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">

<MenuBar>
  <Menu name="game"><text>&amp;Game</text>
  </Menu>
  <Menu name="settings"><text>&amp;Settings</text>
    <Action name="startplayer" append="save_merge"/>
    <Action name="player1" append="save_merge"/>
    <Action name="player2" append="save_merge"/>
    <Separator append="save_merge"/>
    <Action name="theme" append="save_merge"/>
    <Action name="select_carddeck" append="save_merge"/>
    <Separator append="save_merge"/>
    <Action name="change_names" append="save_merge"/>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" hidden="true"><text>Main Toolbar</text>
  <Action name="game_new"/>
  <Action name="game_end"/>
</ToolBar>

</gui>
