/*
    This file is part of the KDE games lskat program
    SPDX-FileCopyrightText: 2006 Martin Heni <kde@heni-online.de>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef LSKATGLOBAL_H
#define LSKATGLOBAL_H

/** Global debug flag */
extern int global_debug;
/** Global skip intro flag */
extern bool global_skip_intro;
/** Demo or autoplay mode */
extern bool global_demo_mode;

#endif
